/****************************************************************************

	File    : sample\sample.c  

	Project : server
	Version : 3.7
	Date    : Sat, 22, Sep 2018 

	Author  : Zheltikov I.A.

****************************************************************************/

#include "sample/sample.h"
#include "sample/sample_srr.h"
#include "server/server.h"
#include <err.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/neutrino.h>


int coid_copy;

int timer;

static int rcvid_del = -1;

// ---------------------------------------------------------> InitSample <---

void InitSample(bool flag, int param)
{
	InitServer ();

	if (flag)
		printf ("Sample() flag is set\n");

	printf ("Sample() param value=%d\n", param);
}

// --------------------------------------------------------> CloseSample <---

void CloseSample(void)
{
	CloseServer ();
}

// ----------------------------------------------------------------> Cmd <---

int Cmd(void)
{
	printf ("Command without data\n");

	return EOK;
}

// -----------------------------------------------------------------> In <---

int In(int in)
{
	printf ("In()  in=%d\n", in);

	return EOK;
}

// ----------------------------------------------------------------> Out <---

int Out(int* out)
{
	*out = 1;

	printf ("Out()  out=%d\n", *out);

	return EOK;
}

// --------------------------------------------------------------> InOut <---

int InOut(int in, int* out)
{
	*out = 2;

	printf ("Inout()  in=%d   out=%d\n", in, *out);

	return ENOSYS; // for example
}

// ------------------------------------------------------------> Delayed <---

int Delayed(int in, int rcvid)
{
	rcvid_del = rcvid;

	printf ("Delayed()  in=%d\n", in);

	return -1;
}

// ------------------------------------------------------------> Vectors <---

int Vectors(int in_len, int out_len, int rcvid)
{
	char* in;
	char out [100];
	int len;
	iov_t iov [2];
	sa_p_vectors_t pmsg;

	in = (char*) malloc (in_len);
	MsgRead (rcvid, in, in_len, sizeof (sa_r_vectors_t));

	printf ("Vectors () in_len=%d   max out_len=%d\n", in_len, out_len);
	printf ("In str '%s'\n", in);

	strcpy (out, "Thanks, well!");
	len = strlen (out) + 1;
	pmsg.out_len = len;

	SETIOV (iov + 0, &pmsg, sizeof (sa_p_vectors_t));
	SETIOV (iov + 1, out, len);
	MsgReplyv (rcvid, EOK, iov, 2);

	free (in);

	return -1;
}

// ---------------------------------------------------------------> Self <---

void Self(void)
{
	printf ("Self()\n");
}

// --------------------------------------------------------------> Pulse <---

void Pulse(int value)
{
	printf ("Pulse()  value=%d\n", value);
}

// --------------------------------------------------------------> Timer <---

void Timer(void)
{
	int value;
	sa_p_delayed_t lmsg;
	static int counter = 0;

	printf ("Timer()\n");

	if (rcvid_del != -1)
		counter++;

	if (counter == 5) {
		MsgSendPulse (coid_copy, SIGEV_PULSE_PRIO_INHERIT, sa_c_self, 0);
		return;
	}

	if (counter == 7) {
		lmsg.out = 5;
		printf ("Timer() delayed out=%d\n", lmsg.out);
		MsgReply (rcvid_del, EOK, &lmsg, sizeof (sa_p_delayed_t));
		return;
	}

	if (counter == 10) {
		value = 15;
		printf ("Timer() delivery event value=%d\n", value);
		DeliverEvent (value);
		return;
	}
}

//===========================================================================

