/****************************************************************************

	File    : sample\sample_ser.c  

	Project : server
	Version : 3.7
	Date    : Sat, 22, Sep 2018 

	Author  : Zheltikov I.A.

****************************************************************************/

#include "sample/sample_ser.h"
#include "sample/sample.h"
#include "sample/sample_opt.h"
#include "sample/sample_srr.h"
#include "server/server_ser.h"
#include <errno.h>
#include <err.h>
#include <sys/dispatch.h>



// --------------------------------------------------> sa_Initialization <---

void sa_Initialization(int argc, char** argv, const char** path
, unsigned int* flags, int* rsize, int* psize)
{

	parms_t parms;

							//    
	OptProc (argc, argv, &parms);

							//   
	InitSample (parms.flag, parms.param);

							//    
	*path  = (const char*) SAMPLE_NAME;
	*flags = 0; // NAME_FLAG_ATTACH_GLOBAL;
	*rsize = sizeof (sa_r_msg_t);
	*psize = sizeof (sa_p_msg_t);
}

// ------------------------------------------------------> sa_Completing <---

void sa_Completing(void)
{
	timer_delete (timer);

	if (pp) {
		CloseSample ();
		pp = false;
	}

	se_Completing ();
}

// -------------------------------------------------> sa_ChannelDepended <---

void sa_ChannelDepended(int coid)
{
	int rc;
	sigevent_t event;
	struct itimerspec itime;

	SIGEV_PULSE_INIT (&event, coid, SIGEV_PULSE_PRIO_INHERIT, sa_c_timer, 0);

	rc = timer_create (CLOCK_REALTIME, &event, &timer);
							if (rc == -1) err (1,
							"Can not create timer");

	itime.it_value.tv_sec     = 0;
	itime.it_value.tv_nsec    = 1; 
	itime.it_interval.tv_sec  = 1;
	itime.it_interval.tv_nsec = 0;
	timer_settime (timer, 0, &itime, NULL);

	coid_copy = coid;

	se_ChannelDepended (coid);
}

// ------------------------------------------------------> sa_ProcessMsg <---

int sa_ProcessMsg(int rcvid, void * r_msg, void * p_msg, int* psize
, struct _msg_info* info)
{
	int rc;
	uint16_t type;

	type = ((se_r_cmd_t*) r_msg)->type;

	rc = ENOSYS;
	*psize = 0;

	switch (type) {

	// --->

	case sa_t_cmd:
		rc = Cmd ();
		break;

	case sa_t_in: {
		sa_r_in_t* rmsg = (sa_r_in_t*) r_msg;
		rc = In (rmsg->in);
		break;
	}

	case sa_t_out: {
		sa_p_out_t* pmsg = (sa_p_out_t*) p_msg;
		rc = Out (&pmsg->out);
		*psize = sizeof (sa_p_out_t);
		break;
	}

	case sa_t_inout: {
		sa_r_inout_t* rmsg = (sa_r_inout_t*) r_msg;
		sa_p_inout_t* pmsg = (sa_p_inout_t*) p_msg;
		rc = InOut (rmsg->in, &pmsg->out);
		*psize = sizeof (sa_p_inout_t);
		break;
	}

	case sa_t_delayed: {
		sa_r_delayed_t* rmsg = (sa_r_delayed_t*) r_msg;
		rc = Delayed (rmsg->in, rcvid);
		break;
	}

	case sa_t_vectors: {
		sa_r_vectors_t* rmsg = (sa_r_vectors_t*) r_msg;
		rc = Vectors (rmsg->in_len, rmsg->out_len, rcvid);
		break;
	}

	// <---    

	default:
		rc = se_ProcessMsg (rcvid, r_msg, p_msg, psize, info);
		break;
	}

	return rc;
}

// ----------------------------------------------------> sa_ProcessPulse <---

void sa_ProcessPulse(pulse_t* pulse, struct _msg_info* info)
{
	switch (pulse->code) {

	// --->

	case sa_c_self:
		Self ();
		break;

	case sa_c_pulse:
		Pulse (pulse->value.sival_int);
		break;

	case sa_c_timer:
		Timer ();
		break;

	// <---

	default:
		se_ProcessPulse(pulse, info);
		break;
	}
}

//===========================================================================

