/****************************************************************************

	File    : server\server_fix.c  

	Project : server
	Version : 3.7
	Date    : Sat, 22, Sep 2018 

	Copynull (0) Zheltikov I.A.

****************************************************************************/

#include "server/server_fix.h"
#include <err.h>
#include <stdlib.h>


bool is_srr_running = true;

void (*Initialization) (int, char**, const char**, unsigned*, int*, int*)
 = NULL;

void (*Completing) () = NULL;

void (*ChannelDepended) (int) = NULL;

int (*ProcessMsg) (int, void*, void*, int*, struct _msg_info*) = NULL;

void (*ProcessPulse) (pulse_t*, struct _msg_info*) = NULL;

//  
static const char* path = NULL;

//   ,   
static unsigned int flags = 0;

//  
static void * r_msg = NULL;

//  
static void * p_msg = NULL;

//   
static name_attach_t* attach = NULL;

//  
static int coid = -1;

//  

static void Prepare(const char* path, unsigned int flags, int rsize, int
 psize, void ** r_msg, void ** p_msg, int* coid, int* chid);

//      
static void MainLoop(void * r_msg, void * p_msg, int chid, int rsize);

// ----------------------------------------------------------------> Run <---

void Run(int argc, char** argv)
{
	int rsize;
	int psize;
	int chid;

	Initialization (argc, argv, &path, &flags, &rsize, &psize);
	Prepare (path, flags, rsize, psize, &r_msg, &p_msg, &coid, &chid);
	ChannelDepended (coid);
	MainLoop (r_msg, p_msg, chid, rsize);
}

// ------------------------------------------------> ServerFixCompleting <---

void ServerFixCompleting(void)
{
	if (coid != -1)
		ConnectDetach (coid);

	if (attach != NULL)
		name_detach (attach, flags);

	if (p_msg != NULL)
		free (p_msg);

	if (r_msg != NULL)
		free (r_msg);
}

// ------------------------------------------------------------> Prepare <---

static void Prepare(const char* path, unsigned int flags, int rsize, int psize
, void ** r_msg, void ** p_msg, int* coid, int* chid)
{

	if (Initialization  == NULL ||
	    Completing      == NULL ||
	    ChannelDepended == NULL || 
	    ProcessMsg      == NULL ||
	    ProcessPulse    == NULL)
		err (EXIT_FAILURE, "Callback is not implemented");

							//    
	*r_msg = malloc (rsize);
							if (*r_msg == NULL) err (EXIT_FAILURE,
							"Can not allocate memory for a message");

	*p_msg = malloc (psize);
							if (*p_msg == NULL) err (EXIT_FAILURE,
							"Can not allocate memory for a message");

							//     
	attach = name_attach (NULL, path, flags);
							if (attach == NULL) err (EXIT_FAILURE,
							"Can not attach the name '%s'", path);

	*coid = ConnectAttach (0, 0, attach->chid, _NTO_SIDE_CHANNEL, 0);
							if (*coid == -1) err (EXIT_FAILURE,
							"Can not establish a connection to self");

	*chid = attach->chid;
}

// -----------------------------------------------------------> MainLoop <---

static void MainLoop(void * r_msg, void * p_msg, int chid, int rsize)
{
	int rc;
	int rcvid;
	int psize;
	int status;
	struct _msg_info info;

							//      
	while (is_srr_running) {
		rcvid = MsgReceive (chid, r_msg, rsize, &info);
							if (rcvid == -1) {
								warn ("MsgReceive error");
								continue;
							}

		if (rcvid != 0) {
			status = ProcessMsg (rcvid, r_msg, p_msg, &psize, &info);

							// ,   status   (-1)
			if (status != -1) {
				rc = MsgReply (rcvid, status, p_msg, psize);
							if (rc == -1)
								warn ("MsgReply error");
			}
		} else
			ProcessPulse ((pulse_t*) r_msg, &info);
	}
}

//===========================================================================

