/****************************************************************************

	File    : Server\SServer.cpp  

	Project : Server
	Version : 3.7
	Date    : Sat, 22, Sep 2018  

	Copynull (0) Zheltikov I.A.

****************************************************************************/

#include "Server/SServer.h"
#include "Server/PServer.h"
#include "Server/OServer.h"
#include <err.h>
#include <errno.h>

SServer::SServer()
{
	pp = NULL;
}

SServer::~SServer()
{
}

// -----------------------------------------------------> Initialization <---

void SServer::Initialization(int argc, char** argv, const char** path
, unsigned int* flags, int* rsize, int* psize)
{

	OServer::parms_t parms;
	OServer opt;

							//    
	opt.OptProc (argc, argv, &parms);

							//   
	pp = new PServer ();

							//    
	*path  = (const char*) SERVER_NAME;
	*flags = 0;
	*rsize = sizeof (se_r_msg_t);
	*psize = sizeof (se_p_msg_t);
}

// ---------------------------------------------------------> Completing <---

void SServer::Completing()
{
	if (pp != NULL) {
		delete (PServer*) pp;
		pp = NULL;
	}

	ServerFixCompleting ();
}

// ----------------------------------------------------> ChannelDepended <---

void SServer::ChannelDepended(int coid)
{
}

// ---------------------------------------------------------> ProcessMsg <---

int SServer::ProcessMsg(int rcvid, void * r_msg, void * p_msg, int* psize
, struct _msg_info* info)
{
	int rc;
	uint16_t type;
	PServer* p;

	p    = (PServer*) pp;
	type = ((se_r_cmd_t*) r_msg)->type;

	rc = ENOSYS;
	*psize = 0;

	switch (type) {

	case se_t_quit:
		rc = p->Complete (&is_srr_running);
		break;

	case se_t_evdlvr: {
		se_r_evdlvr_t* rmsg = (se_r_evdlvr_t*) r_msg;
		rc = p->EventForDelivering (rcvid, &(rmsg->event));
		break;
	}

	case _IO_CONNECT:
	case _IO_CLOSE:
		rc = EOK;
		break;

	default:
		warnx ("unsupported command type=%d", type);
		rc = ENOSYS;
		break;
	}

	return rc;
}

// -------------------------------------------------------> ProcessPulse <---

void SServer::ProcessPulse(pulse_t* pulse, struct _msg_info* info)
{
	PServer* p;

	p = (PServer*) pp;

	switch (pulse->code) {

	case se_c_quit:
		p->CompleteByPulse (&is_srr_running);
		break;

	case _PULSE_CODE_DISCONNECT:
		ConnectDetach (pulse->scoid);
		break;

	case _PULSE_CODE_UNBLOCK:
	case _PULSE_CODE_THREADDEATH:
	case _PULSE_CODE_COIDDEATH:
	case _PULSE_CODE_NET_ACK:
	case _PULSE_CODE_NET_UNBLOCK:
	case _PULSE_CODE_NET_DETACH:
		break;

	default:
		warnx ("unsupported pulse code=%d", pulse->code);
		break;
	}
}


//===========================================================================

